from memx.memory import BaseMemory


class RedisMemory(BaseMemory):
    def __init__(self, uri: str):
        raise NotImplementedError("RedisMemory is not implemented yet")

    def add(self, messages: list[dict]):
        pass

    def get(self) -> list[dict]:
        pass



from typing import TYPE_CHECKING, Union

from sqlalchemy import text

from memx.models.sql import SQLEngineConfig

if TYPE_CHECKING:
    from memx.engine.postgres import PostgresEngine
    from memx.engine.sqlite import SQLiteEngine


async def get_session(
    sql_engine: Union["PostgresEngine", "SQLiteEngine"], id: str
) -> SQLEngineConfig | None:
    """."""

    async with sql_engine.AsyncSession() as session:
        result = (
            await session.execute(
                text(sql_engine.get_session_sql),
                {"session_id": id},
            )
        ).first()

    if result[0] == 1:  # type: ignore
        engine_config = SQLEngineConfig(
            table=sql_engine.table_name,
            add_query=sql_engine.add_sql,
            get_query=sql_engine.get_sql,
        )
        return engine_config


def get_session_sync(
    sql_engine: Union["PostgresEngine", "SQLiteEngine"], id: str
) -> SQLEngineConfig | None:
    """."""

    with sql_engine.SyncSession() as session:
        result = session.execute(text(sql_engine.get_session_sql), {"session_id": id}).first()

    if result[0] == 1:  # type: ignore
        engine_config = SQLEngineConfig(
            table=sql_engine.table_name,
            add_query=sql_engine.add_sql,
            get_query=sql_engine.get_sql,
        )
        return engine_config



from abc import ABC, abstractmethod


class BaseMemory(ABC):
    @abstractmethod
    def add(self, messages: list[dict]):
        pass

    @abstractmethod
    def get(self) -> list[dict]:
        pass

    def get_id(self) -> str:
        return self._session_id  # type: ignore



from memx.engine import BaseEngine
from memx.memory.redis import RedisMemory


class RedisEngine(BaseEngine):
    def __init__(self, uri: str):
        raise NotImplementedError("RedisEngine is not implemented yet")
        pass

    def create_session(self) -> RedisMemory:
        pass

    def get_session(self, session_id: str) -> RedisMemory | None:
        pass



from textwrap import dedent

from sqlalchemy import create_engine, text
from sqlalchemy.ext.asyncio import AsyncSession, async_sessionmaker, create_async_engine
from sqlalchemy.orm import Session, sessionmaker

from memx.engine import BaseEngine
from memx.memory.sqlite import SQLiteMemory
from memx.models.sql import SQLEngineConfig
from memx.services import sql_service


class SQLiteEngine(BaseEngine):
    def __init__(self, uri: str, table: str, start_up: bool = False):
        """Initialize SQLite engine for memory storage.

        Args:
            uri: Database connection URI for SQLite.
            table: Name of the table to use for storing memories.
            start_up: If True, create the table if it doesn't exist (blocking operation).
        """

        self.table_name = f"'{table.strip()}'"
        self._init_queries()

        self.async_engine = create_async_engine(uri, echo=False, future=True)
        self.AsyncSession = async_sessionmaker(
            bind=self.async_engine,
            expire_on_commit=False,
            class_=AsyncSession,
        )

        drivers, others = uri.split(":", 1)  # type: ignore[reportUnusedVariable]
        self.sync_engine = create_engine(
            f"sqlite:{others}",
            echo=False,
            connect_args={"check_same_thread": True},
        )

        self.SyncSession = sessionmaker(
            autocommit=False,
            autoflush=False,
            bind=self.sync_engine,
            class_=Session,
        )

        self.sync = _sync(self)

        if start_up:
            self.start_up()  # blocking operation

    def create_session(self) -> SQLiteMemory:
        """Create a local memory session."""

        engine_config = SQLEngineConfig(
            table=self.table_name,
            add_query=self.add_sql,
            get_query=self.get_sql,
        )
        return SQLiteMemory(self.AsyncSession, self.SyncSession, engine_config)

    async def get_session(self, id: str) -> SQLiteMemory | None:
        """Get a memory session."""

        if engine_config := await sql_service.get_session(self, id):
            return SQLiteMemory(self.AsyncSession, self.SyncSession, engine_config, id)

        return None  # explicit is better than implicit

    def _init_queries(self):
        """."""

        self.table_sql = dedent(f"""
            CREATE TABLE IF NOT EXISTS {self.table_name} (
                session_id TEXT,
                message JSONB,
                created_at TIMESTAMP DEFAULT CURRENT_TIMESTAMP
            );
            CREATE INDEX IF NOT EXISTS session_index ON {self.table_name} (session_id);
        """)

        self.add_sql = dedent(f"""
            INSERT INTO {self.table_name} (session_id, message, created_at)
            VALUES (:session_id, :message, :created_at);
        """)

        self.get_sql = dedent(f"""
            SELECT message FROM {self.table_name}
            WHERE session_id = :session_id
            ORDER BY created_at ASC;
        """)

        self.get_session_sql = dedent(f"""
            SELECT EXISTS(
                SELECT 1 FROM {self.table_name}
                WHERE session_id=:session_id
            ) as r;
        """)

    def start_up(self):
        """Create the table if it doesn't exist."""

        with self.sync_engine.begin() as conn:
            conn.connection.executescript(self.table_sql)


class _sync:
    def __init__(self, parent: "SQLiteEngine"):
        self.pe = parent

    def get_session(self, id: str) -> SQLiteMemory | None:
        """Get a memory session."""

        if engine_config := sql_service.get_session_sync(self.pe, id):
            return SQLiteMemory(self.pe.AsyncSession, self.pe.SyncSession, engine_config, id)

        return None  # explicit is better than implicit



from pymongo import AsyncMongoClient, MongoClient
from pymongo.server_api import ServerApi

from memx.engine import BaseEngine
from memx.memory.mongodb import MongoDBMemory


class MongoDBEngine(BaseEngine):
    def __init__(self, uri: str, database: str, collection: str):
        """MongoDB memory engine."""

        self.client = MongoClient(uri)
        self.async_client = AsyncMongoClient(
            uri,
            server_api=ServerApi(version="1", strict=True, deprecation_errors=True),
        )

        self.db = self.client[database]
        self.async_db = self.async_client.get_database(database)

        self.sync_collection = self.db[collection]
        self.async_collection = self.async_db[collection]

        self.sync = _sync(self)

    def create_session(self) -> MongoDBMemory:
        return MongoDBMemory(self.async_collection, self.sync_collection)

    async def get_session(self, id: str) -> MongoDBMemory | None:
        result = await self.async_collection.find_one({"session_id": id})

        if result:
            return MongoDBMemory(self.async_collection, self.sync_collection, id)


class _sync:
    def __init__(self, parent: "MongoDBEngine"):
        self.pe = parent

    def get_session(self, id: str) -> MongoDBMemory | None:
        """Get a memory session."""

        result = self.pe.sync_collection.find_one({"session_id": id})

        if result:
            return MongoDBMemory(self.pe.async_collection, self.pe.sync_collection, id)



from typing_extensions import TypeAliasType

# NOTE: probably we shouldn't include list[JSON]
# In summary; anything that can be serialized with orjson.dumps()
JSON = TypeAliasType("JSON", "dict[str, JSON] | list[JSON] | str | int | float | bool | None")



from abc import ABC, abstractmethod

from memx.memory import BaseMemory


class BaseEngine(ABC):
    @abstractmethod
    def create_session(self) -> BaseMemory:
        """Create a memory session."""
        pass

    @abstractmethod
    def get_session(self, session_id: str) -> BaseMemory | None:
        """Get a memory session from backend."""
        pass



import tempfile
from pathlib import Path

from memx.engine.sqlite import SQLiteEngine


def test_engine_init():
    sqlite_uri = "sqlite+aiosqlite:///:memory:"
    engine = SQLiteEngine(sqlite_uri, "test-messages", start_up=True)
    assert engine.table_name == "'test-messages'"
    assert engine.async_engine is not None
    assert engine.sync_engine is not None
    assert engine.AsyncSession is not None
    assert engine.SyncSession is not None
    assert isinstance(engine.add_sql, str)
    assert isinstance(engine.get_sql, str)
    assert isinstance(engine.get_session_sql, str)


def test_same_session_attributes():
    sqlite_uri = "sqlite+aiosqlite:///:memory:"
    engine1 = SQLiteEngine(sqlite_uri, "test-messages", start_up=True)
    m1 = engine1.create_session()
    m2 = engine1.create_session()

    assert m1.SyncSession == m2.SyncSession
    assert m1.AsyncSession == m2.AsyncSession
    assert m1.engine_config == m2.engine_config


def test_simple_add():
    sqlite_uri = "sqlite+aiosqlite:///:memory:"
    engine = SQLiteEngine(sqlite_uri, "test-messages", start_up=True)
    m1 = engine.create_session()

    messages = [{"role": "user", "content": "Hello, how are you?"}]
    m1.sync.add(messages)

    assert m1.sync.get() == messages

    new_message = [{"role": "agent", "content": "Fine, thanks for asking"}]
    m1.sync.add(new_message)
    messages.extend(new_message)

    assert m1.sync.get() == messages


def test_resume_session():
    sqlite_uri = "sqlite+aiosqlite:///:memory:"
    engine = SQLiteEngine(sqlite_uri, "test-messages", start_up=True)
    m1 = engine.create_session()

    messages = [{"role": "user", "content": "Hello, how are you?"}]
    m1.sync.add(messages)

    assert m1.sync.get() == messages

    m2 = engine.sync.get_session(m1.get_id())

    assert m2.sync.get() == messages  # type: ignore
    assert m1.sync.get() == m2.sync.get()  # type: ignore
    assert engine.sync.get_session("asdf") is None


async def test_simple_add_async():
    fp = tempfile.gettempdir() + "/memx-test.db"
    sqlite_uri = f"sqlite+aiosqlite:///{fp}"
    engine = SQLiteEngine(sqlite_uri, "test-messages", start_up=True)
    m1 = engine.create_session()

    messages = [{"role": "user", "content": "Hello, how are you?"}]
    await m1.add(messages)

    assert await m1.get() == messages

    new_message = [{"role": "agent", "content": "Fine, thanks for asking"}]
    await m1.add(new_message)
    messages.extend(new_message)

    assert await m1.get() == messages

    Path(fp).unlink()


async def test_resume_session_async():
    fp = tempfile.gettempdir() + "/memx-test.db"
    sqlite_uri = f"sqlite+aiosqlite:///{fp}"
    engine = SQLiteEngine(sqlite_uri, "test-messages", start_up=True)
    m1 = engine.create_session()

    messages = [{"role": "user", "content": "Hello, how are you?"}]
    await m1.add(messages)

    assert await m1.get() == messages

    m2 = await engine.get_session(m1.get_id())

    assert await m2.get() == messages  # type: ignore
    assert await m1.get() == await m2.get()  # type: ignore
    assert await engine.get_session("asdf-11!!") is None

    Path(fp).unlink()



from unittest.mock import MagicMock, patch

from memx.engine.postgres import PostgresEngine


@patch("memx.engine.postgres.create_async_engine")
@patch("memx.engine.postgres.create_engine")
@patch("memx.engine.postgres.async_sessionmaker")
@patch("memx.engine.postgres.sessionmaker")
def test_engine_init(
    mock_sessionmaker,
    mock_async_sessionmaker,
    mock_create_engine,
    mock_create_async_engine,
):
    # Setup mocks
    mock_async_engine = MagicMock()
    mock_sync_engine = MagicMock()
    mock_async_session = MagicMock()
    mock_sync_session = MagicMock()

    mock_create_async_engine.return_value = mock_async_engine
    mock_create_engine.return_value = mock_sync_engine
    mock_async_sessionmaker.return_value = mock_async_session
    mock_sessionmaker.return_value = mock_sync_session

    # Mock the start_up method to avoid actual database operations
    with patch.object(PostgresEngine, "start_up", return_value=None):
        pg_uri = "postgresql+psycopg://admin:1234@localhost:5433/test-database"
        engine = PostgresEngine(pg_uri, "memx-messages", start_up=True)

        assert engine.table_name == '"memx-messages"'
        assert engine.async_engine is not None
        assert engine.sync_engine is not None
        assert engine.AsyncSession is not None
        assert engine.SyncSession is not None
        assert isinstance(engine.table_sql, str)
        assert isinstance(engine.add_sql, str)
        assert isinstance(engine.get_sql, str)
        assert isinstance(engine.get_session_sql, str)

        # Verify that engines were created with correct URI
        mock_create_async_engine.assert_called_once()
        mock_create_engine.assert_called_once()
        mock_async_sessionmaker.assert_called_once()
        mock_sessionmaker.assert_called_once()



from datetime import UTC, datetime
from uuid import uuid4

import orjson
from sqlalchemy import text
from sqlalchemy.ext.asyncio import AsyncSession, async_sessionmaker
from sqlalchemy.orm import Session, sessionmaker

from memx.memory import BaseMemory
from memx.models.sql import SQLEngineConfig
from memx.utils import JSON


class PostgresMemory(BaseMemory):
    def __init__(
        self,
        async_session_maker: async_sessionmaker[AsyncSession],  # type: ignore
        sync_session_maker: sessionmaker[Session],
        engine_config: SQLEngineConfig,
        session_id: str = None,
    ):
        self.AsyncSession = async_session_maker
        self.SyncSession = sync_session_maker

        self.engine_config = engine_config

        self.sync = _sync(self)  # to group sync methods

        if session_id:
            self._session_id = session_id
        else:
            self._session_id = str(uuid4())

    async def add(self, messages: list[JSON]):
        # TODO: refactor this with sqlite
        await self._pre_add()

        ts_now = datetime.now(UTC)
        data = {
            "session_id": self._session_id,
            "message": orjson.dumps(messages).decode("utf-8"),
            "updated_at": ts_now,
        }

        async with self.AsyncSession() as session:
            await session.execute(text(self.engine_config.add_query), data)
            await session.commit()

    async def get(self) -> list[JSON]:
        async with self.AsyncSession() as session:
            result = await session.execute(
                text(self.engine_config.get_query),
                {"session_id": self._session_id},
            )

        result = result.first()
        result = getattr(result, "message", [])

        return result

    async def _pre_add(self):
        pass


class _sync(BaseMemory):
    def __init__(self, parent: "PostgresMemory"):
        self.pm = parent  # parent memory (?)

    def add(self, messages: list[JSON]):
        # TODO: refactor this with sqlite

        self._pre_add()

        ts_now = datetime.now(UTC)
        data = {
            "session_id": self.pm._session_id,
            "message": orjson.dumps(messages).decode("utf-8"),
            "updated_at": ts_now,
        }

        with self.pm.SyncSession() as session:
            session.execute(text(self.pm.engine_config.add_query), data)
            session.commit()

    def get(self) -> list[JSON]:
        with self.pm.SyncSession() as session:
            result = session.execute(
                text(self.pm.engine_config.get_query),
                {"session_id": self.pm._session_id},
            )

        result = result.first()
        result = getattr(result, "message", [])

        return result

    def _pre_add(self):
        pass



from textwrap import dedent

from sqlalchemy import create_engine, text
from sqlalchemy.ext.asyncio import AsyncSession, async_sessionmaker, create_async_engine
from sqlalchemy.orm import Session, sessionmaker

from memx.engine import BaseEngine
from memx.memory.postgres import PostgresMemory
from memx.models.sql import SQLEngineConfig
from memx.services import sql_service


class PostgresEngine(BaseEngine):
    def __init__(self, uri: str, table: str, schema: str = "public", start_up: bool = None):
        """Initialize PostgreSQL engine for memory storage.

        Args:
            uri: Database connection URI for PostgreSQL (must use 'postgresql+psycopg' driver).
            table: Name of the table to use for storing memories.
            schema: Database schema name (defaults to 'public').
            start_up: If True, create the table if it doesn't exist (blocking operation).
        """

        self.table_name = f'"{table.strip()}"'
        self._init_queries()

        driver, _ = uri.split(":", 1)
        if driver.strip() != "postgresql+psycopg":
            raise ValueError("For the moment, only 'postgresql+psycopg' driver is supported")

        common_args = {
            "autocommit": False,
            "autoflush": False,
            "expire_on_commit": True,
        }

        self.async_engine = create_async_engine(
            uri,
            connect_args={"options": f"-csearch_path={schema}"},
        )
        self.AsyncSession = async_sessionmaker(
            **common_args,
            bind=self.async_engine,
            class_=AsyncSession,
        )  # type: ignore

        self.sync_engine = create_engine(
            uri,
            connect_args={"options": f"-csearch_path={schema}"},
        )
        self.SyncSession = sessionmaker(
            **common_args,
            bind=self.sync_engine,
            class_=Session,
        )  # type: ignore

        self.sync = _sync(self)

        if start_up:
            self.start_up()  # blocking operation

    def create_session(self) -> PostgresMemory:
        """Get or create a memory session."""

        engine_config = SQLEngineConfig(
            table=self.table_name,
            add_query=self.add_sql,
            get_query=self.get_sql,
        )
        return PostgresMemory(self.AsyncSession, self.SyncSession, engine_config)

    async def get_session(self, id: str) -> PostgresMemory | None:
        """Get a memory session."""

        if engine_config := await sql_service.get_session(self, id):
            return PostgresMemory(self.AsyncSession, self.SyncSession, engine_config, id)

        return None  # explicit is better than implicit

    def _init_queries(self):
        """."""

        self.table_sql = dedent(f"""
            CREATE TABLE IF NOT EXISTS {self.table_name} (
                session_id uuid PRIMARY KEY,
                message JSONB,
                created_at TIMESTAMP WITH TIME ZONE DEFAULT (NOW() AT TIME ZONE 'UTC'),
                updated_at TIMESTAMP WITH TIME ZONE DEFAULT (NOW() AT TIME ZONE 'UTC')
            );
        """)

        self.add_sql = dedent(f"""
            INSERT INTO {self.table_name} (session_id, message, updated_at)
            VALUES (:session_id, cast(:message as jsonb), :updated_at)
            ON CONFLICT (session_id)
            DO UPDATE SET
                message = COALESCE({self.table_name}.message, '[]'::jsonb) || EXCLUDED.message,
                updated_at = EXCLUDED.updated_at;
        """)

        self.get_sql = dedent(f"""
            SELECT * FROM {self.table_name}
            WHERE session_id = :session_id;
        """)

        self.get_session_sql = dedent(f"""
            SELECT EXISTS(
                SELECT 1 FROM {self.table_name}
                WHERE session_id=:session_id
            ) as r;
        """)

    def start_up(self):
        """Create the table if it doesn't exist."""

        with self.SyncSession() as session:
            session.execute(text(self.table_sql))
            session.commit()


class _sync:
    def __init__(self, parent: "PostgresEngine"):
        self.pe = parent

    def get_session(self, id: str) -> PostgresMemory | None:
        """Get a memory session."""

        if engine_config := sql_service.get_session_sync(self.pe, id):
            return PostgresMemory(self.pe.AsyncSession, self.pe.SyncSession, engine_config, id)

        return None  # explicit is better than implicit



from datetime import UTC, datetime
from uuid import uuid4

import orjson
from sqlalchemy import Result, text
from sqlalchemy.ext.asyncio import AsyncSession, async_sessionmaker
from sqlalchemy.orm import Session, sessionmaker

from memx.memory import BaseMemory
from memx.models.sql import SQLEngineConfig
from memx.utils import JSON


class SQLiteMemory(BaseMemory):
    def __init__(
        self,
        async_session_maker: async_sessionmaker[AsyncSession],  # type: ignore
        sync_session_maker: sessionmaker[Session],
        engine_config: SQLEngineConfig,
        session_id: str = None,
    ):
        self.AsyncSession = async_session_maker
        self.SyncSession = sync_session_maker

        self.engine_config = engine_config

        self.sync = _sync(self)  # to group sync methods

        if session_id:
            self._session_id = session_id
        else:
            self._session_id = str(uuid4())

    async def add(self, messages: list[JSON]):
        await self._pre_add()

        data = self._format_messages(messages)

        async with self.AsyncSession() as session:
            await session.execute(text(self.engine_config.add_query), data)
            await session.commit()

    async def get(self) -> list[JSON]:
        async with self.AsyncSession() as session:
            result = await session.execute(
                text(self.engine_config.get_query),
                {"session_id": self._session_id},
            )

        messages = _merge_messages(result)

        return messages

    async def _pre_add(self):
        pass

    def _format_messages(self, messages: list[JSON]) -> dict:
        ts_now = datetime.now(UTC).isoformat()
        data = {
            "session_id": self._session_id,
            "message": orjson.dumps(messages).decode("utf-8"),
            "created_at": ts_now,
        }

        return data


class _sync(BaseMemory):
    def __init__(self, parent: "SQLiteMemory"):
        self.pm = parent  # parent memory (?)

    def add(self, messages: list[JSON]):
        self._pre_add()

        data = self.pm._format_messages(messages)

        with self.pm.SyncSession() as session:
            session.execute(text(self.pm.engine_config.add_query), data)
            session.commit()

    def get(self) -> list[JSON]:
        with self.pm.SyncSession() as session:
            result = session.execute(
                text(self.pm.engine_config.get_query),
                {"session_id": self.pm._session_id},
            )

        messages = _merge_messages(result)

        return messages  # type: ignore

    def _pre_add(self):
        pass


def _merge_messages(msg_result: Result) -> list[JSON]:
    """Merge messages from the result of the query."""

    # list.extend is the fastest approach
    result = [dict(row._mapping) for row in msg_result.fetchall()]
    messages = []

    for r in result:
        messages.extend(orjson.loads(r["message"]))

    return messages



from datetime import UTC, datetime
from uuid import uuid4

from pymongo.asynchronous.collection import AsyncCollection
from pymongo.collection import Collection

from memx.memory import BaseMemory


class MongoDBMemory(BaseMemory):
    def __init__(
        self,
        async_collection: AsyncCollection,
        sync_collection: Collection,
        session_id: str = None,
    ):
        self.async_collection = async_collection
        self.sync_collection = sync_collection

        self.sync = _sync(self)  # to group sync methods

        if session_id:
            self._session_id = session_id
        else:
            self._session_id = str(uuid4())

    async def add(self, messages: list[dict]):
        ts_now = datetime.now(UTC)

        await self.async_collection.find_one_and_update(
            {"session_id": self._session_id},
            {
                "$push": {"messages": {"$each": messages}},
                "$setOnInsert": {"created_at": ts_now},
                "$set": {"updated_at": ts_now},
            },
            upsert=True,
        )

    async def get(self) -> list[dict]:
        doc = await self.async_collection.find_one({"session_id": self._session_id})

        return (doc or {}).get("messages", [])


class _sync(BaseMemory):
    def __init__(self, parent: "MongoDBMemory"):
        self.pm = parent  # parent memory (?)

    def add(self, messages: list[dict]):
        ts_now = datetime.now(UTC)

        self.pm.sync_collection.find_one_and_update(
            {"session_id": self.pm._session_id},
            {
                "$push": {"messages": {"$each": messages}},
                "$setOnInsert": {"created_at": ts_now},
                "$set": {"updated_at": ts_now},
            },
            upsert=True,
        )

    def get(self) -> list[dict]:
        doc = self.pm.sync_collection.find_one({"session_id": self.pm._session_id})

        return (doc or {}).get("messages", [])



import asyncio
import pickle
from pathlib import Path
from uuid import uuid4

import aiofiles
from typing_extensions import deprecated

from memx.memory import BaseMemory


@deprecated("Use SQLiteMemory instead")
class DiskMemory(BaseMemory):
    def __init__(self, session_id: str = None, dir: str = None):
        file_id = session_id if session_id else str(uuid4())
        file_dir = Path(dir) if dir else Path.home() / ".memx"

        self.file_path = Path(file_dir) / f"{file_id}.pkl"
        file_dir.mkdir(parents=True, exist_ok=True)

        if not self.file_path.is_file():
            with open(self.file_path, "wb") as f:
                pickle.dump([], f, protocol=pickle.HIGHEST_PROTOCOL)

        self._session_id = file_id

        self.sync = _sync(self)  # to group sync methods

    async def add(self, messages: list[dict]):
        # read the file
        async with aiofiles.open(self.file_path, "rb") as f:
            pickled_data = await f.read()

        stored_messages: list[dict] = await asyncio.to_thread(pickle.loads, pickled_data)

        # extend the messages
        stored_messages.extend(messages)

        # write the file
        pickled_data = await asyncio.to_thread(
            pickle.dumps, stored_messages, protocol=pickle.HIGHEST_PROTOCOL
        )

        async with aiofiles.open(self.file_path, "wb") as f:
            await f.write(pickled_data)

    async def get(self) -> list[dict]:
        async with aiofiles.open(self.file_path, "rb") as f:
            pickled_data = await f.read()

        stored_messages: list[dict] = await asyncio.to_thread(pickle.loads, pickled_data)

        return stored_messages


class _sync(BaseMemory):
    def __init__(self, parent: "DiskMemory"):
        self.pm = parent  # parent memory (?)

    def add(self, messages: list[dict]):
        with open(self.pm.file_path, "rb") as f:
            stored_messages: list[dict] = pickle.load(f)

        stored_messages.extend(messages)

        with open(self.pm.file_path, "wb") as f:
            pickle.dump(stored_messages, f, protocol=pickle.HIGHEST_PROTOCOL)

    def get(self) -> list[dict]:
        with open(self.pm.file_path, "rb") as f:
            stored_messages: list[dict] = pickle.load(f)

        return stored_messages


@deprecated("Use SQLiteMemory with :memory: URI instead")
class InMemory(BaseMemory):
    # TODO: add .sync just for consistency
    def __init__(self, session_id: str = None):
        global __memx_in_memory__
        if "__memx_in_memory__" not in globals().keys():
            __memx_in_memory__ = {}

        if session_id:
            self._messages = __memx_in_memory__[session_id]
            _session_id = session_id
        else:
            _session_id = str(uuid4())
            __memx_in_memory__[_session_id] = []
            self._messages = __memx_in_memory__[_session_id]

        self._session_id = _session_id

    def add(self, messages: list[dict]):
        self._messages.extend(messages)

    def get(
        self,
    ) -> list[str]:
        return self._messages
